@@ -1,14 +1,18 @@
+import asyncio
+import datetime as dtm
 from decimal import Decimal
+from typing import AsyncGenerator, Dict, List, Optional
+
+import pandas as pd
+
 from pyqqq.brokerage.kis.domestic_stock import KISDomesticStock
 from pyqqq.brokerage.kis.oauth import KISAuth
 from pyqqq.data.realtime import get_all_last_trades
 from pyqqq.datatypes import *
-from pyqqq.utils.market_schedule import get_market_schedule
+from pyqqq.utils.limiter import CallLimiter
+from pyqqq.utils.logger import get_logger
+from pyqqq.utils.market_schedule import get_market_schedule, is_full_day_closed
 from pyqqq.utils.mock_api import with_mock
-from typing import AsyncGenerator, Dict, List, Optional
-import asyncio
-import datetime as dtm
-import pandas as pd
 
 
 class KISStockPosition(StockPosition):
@@ -20,7 +24,6 @@ class KISStockOrder(StockOrder):
 
 
 class KISSimpleDomesticStock:
-
     """
     한국투자증권 국내 주식 API 사용하여 주식 거래를 하기 위한 클래스입니다.
 
@@ -42,11 +45,11 @@ class KISSimpleDomesticStock:
         hts_id: Optional[str] = None,
         corp_data: Optional[dict] = None,
     ):
-
         self.account_no = account_no
         self.account_product_code = account_product_code
         self.stock_api = KISDomesticStock(auth, corp_data)
         self.hts_id = hts_id
+        self.logger = get_logger(__name__ + ".KISSimpleDomesticStock")
 
     def get_account(self) -> dict:
         """
@@ -61,16 +64,12 @@ class KISSimpleDomesticStock:
             - pnl_amount (int): 손익 금액
             - pnl_rate (Decimal): 손익률
         """
-        r = self.stock_api.inquire_balance(self.account_no, self.account_product_code, '01')
-        data = r['output2'][0]
+        r = self.stock_api.inquire_balance(self.account_no, self.account_product_code, "01")
+        data = r["output2"][0]
 
         purchase_amount = int(data["pchs_amt_smtl_amt"])
         pnl_amount = int(data["evlu_pfls_smtl_amt"])
-        pnl_rate = (
-            Decimal(pnl_amount / purchase_amount) * 100
-            if purchase_amount != 0
-            else Decimal(0)
-        )
+        pnl_rate = Decimal(pnl_amount / purchase_amount) * 100 if purchase_amount != 0 else Decimal(0)
 
         result = {
             "total_balance": int(data["tot_evlu_amt"]),
@@ -81,10 +80,12 @@ class KISSimpleDomesticStock:
         }
         return result
 
-    def get_possible_quantity(self,
-                              asset_code: str,
-                              order_type: OrderType = OrderType.MARKET,
-                              price: int = 0) -> dict:
+    def get_possible_quantity(
+        self,
+        asset_code: str,
+        order_type: OrderType = OrderType.MARKET,
+        price: int = 0,
+    ) -> dict:
         """
         주문 가능 수량을 조회합니다.
 
@@ -103,6 +104,7 @@ class KISSimpleDomesticStock:
             - amount (int): 주문 시 소요 금액
         """
 
+        # XXX MID_POINT, STOP_LIMIT 의 경우, 홈페이지에는 안내가 없으나 API 호출은 가능
         def __get_order_type_code():
             if order_type == OrderType.MARKET:
                 return "01"
@@ -116,6 +118,14 @@ class KISSimpleDomesticStock:
                 return "13"
             elif order_type == OrderType.MARKET_FOK:
                 return "14"
+            elif order_type == OrderType.MID_POINT:
+                return "21"
+            elif order_type == OrderType.MID_POINT_IOC:
+                return "23"
+            elif order_type == OrderType.MID_POINT_FOK:
+                return "24"
+            elif order_type == OrderType.STOP_LIMIT:
+                return "22"
             else:
                 raise ValueError("지원하지 않는 주문 유형입니다.")
 
@@ -127,7 +137,7 @@ class KISSimpleDomesticStock:
             price,
         )
 
-        data = resp['output']
+        data = resp["output"]
         result = {
             "investable_cash": data["ord_psbl_cash"],
             "reusable_amount": data["ruse_psbl_amt"],
@@ -157,6 +167,7 @@ class KISSimpleDomesticStock:
                 self.account_no,
                 self.account_product_code,
                 "02",
+                "X",
                 tr_cont=tr_cont,
                 ctx_area_fk100=ctx_area_fk100,
                 ctx_area_nk100=ctx_area_nk100,
@@ -187,21 +198,44 @@ class KISSimpleDomesticStock:
 
         return result
 
-    def get_historical_daily_data(self, asset_code: str, first_date: dtm.date, last_date: dtm.date, adjusted_price: bool = True) -> pd.DataFrame:
+    def _get_data_exchange(self, data_exchange: DataExchange) -> str:
+        if data_exchange == DataExchange.KRX:
+            return "J"
+        elif data_exchange == DataExchange.NXT:
+            return "NX"
+        elif data_exchange == DataExchange.UN:
+            return "UN"
+        else:
+            raise ValueError("지원하지 않는 거래소입니다.")
+
+    def get_historical_daily_data(
+        self,
+        asset_code: str,
+        first_date: dtm.date,
+        last_date: dtm.date,
+        adjusted_price: bool = True,
+        data_exchange: DataExchange = DataExchange.KRX,
+    ) -> pd.DataFrame:
         """
         일봉 데이터 검색
 
+        Note:
+            - 두 거래소에서 공통으로 거래정지된 종목의 시가/고가/저가/종가는 모두 동일하며, 그 외 값은 모두 0 입니다.
+            - NXT 매매체결 종목이 아니거나, NXT 거래소에서 거래가 불가능한 종목을 DataExchange.NXT 또는 DataExchange.UN으로 조회하면, 모든 값은 0 입니다.
+            - 조회 기간 중 휴장일은 포함되지 않습니다.
+
         Args:
             asset_code(str): 종목코드
             first_date(datetime.date): 조회 시작일자
             last_date(datetime.date): 조회 종료일자
             adjusted_price(bool): 수정 주가 여부
+            data_exchange(DataExchange): 데이터 거래소
 
         Returns:
             pd.DataFrame: 일봉 데이터
         """
-        assert first_date <= last_date, 'last_date는 first_date와 같거나, 이후 날짜여야 합니다'
-        assert last_date <= dtm.date.today(), 'last_date는 오늘과 같거나 이전이어야 합니다.'
+        assert first_date <= last_date, "last_date는 first_date와 같거나, 이후 날짜여야 합니다"
+        assert last_date <= dtm.date.today(), "last_date는 오늘과 같거나 이전이어야 합니다."
 
         max_days_per_request = 100
         total_days = (last_date - first_date).days
@@ -212,54 +246,109 @@ class KISSimpleDomesticStock:
             search_start = first_date + dtm.timedelta(days=i)
             search_end = min(first_date + dtm.timedelta(days=i + max_days_per_request), last_date)
 
+            CallLimiter().wait_limit_rate(20, scope="kis/inquire_daily_itemchartprice")
             r = self.stock_api.inquire_daily_itemchartprice(
                 asset_code,
                 search_start,
                 search_end,
-                fid_period_div_code='D',
-                fid_org_adj_prc='0'
-                if adjusted_price else '1')
+                fid_period_div_code="D",
+                fid_cond_mrkt_div_code=self._get_data_exchange(data_exchange),
+                fid_org_adj_prc="0" if adjusted_price else "1",
+            )
 
             chunk = []
-            for item in r['output2']:
+            for item in r["output2"]:
                 if not item:
                     continue
 
-                chunk.append({
-                    'date': item['stck_bsop_date'],
-                    'open': item['stck_oprc'],
-                    'high': item['stck_hgpr'],
-                    'low': item['stck_lwpr'],
-                    'close': item['stck_clpr'],
-                    'volume': item['acml_vol'],
-                })
-            chunk.reverse()
+                chunk.append(
+                    {
+                        "date": item["stck_bsop_date"],
+                        "open": item["stck_oprc"],
+                        "high": item["stck_hgpr"],
+                        "low": item["stck_lwpr"],
+                        "close": item["stck_clpr"],
+                        "volume": item["acml_vol"],
+                        "value": item["acml_tr_pbmn"],
+                    }
+                )
             result.extend(chunk)
 
-        df = pd.DataFrame(result)
-        df['date'] = pd.to_datetime(df['date'])
-        df.set_index('date', inplace=True)
+        df = pd.DataFrame(result, columns=["date", "open", "high", "low", "close", "volume", "value"])
+        df["date"] = pd.to_datetime(df["date"])
+        df.set_index("date", inplace=True)
 
+        # 조회기간 중 데이터가 없는 날짜는 0 으로 채움
+        dates = pd.date_range(start=first_date, end=last_date, freq="B")
+        if len(dates) != len(df):
+            dates = [date for date in dates if not is_full_day_closed(date)]
+            df = df.reindex(dates, fill_value=0)
+
+        df = df.sort_index(ascending=False)
         return df
 
-    def get_today_minute_data(self, asset_code: str) -> pd.DataFrame:
+    def get_today_minute_data(
+        self,
+        asset_code: str,
+        data_exchange: DataExchange = DataExchange.KRX,
+    ) -> pd.DataFrame:
         """
         분봉 데이터 검색
 
+        Note:
+            - 시간외 종가, 시간외 단일가 데이터는 포함되어 있지 않습니다.
+            - 두 거래소에서 공통으로 거래정지된 종목의 시가/고가/저가/종가는 모두 동일하며, 그 외 값은 모두 0 입니다.
+            - NXT 매매체결 종목이 아니거나, NXT 거래소에서 거래가 불가능한 종목을 DataExchange.NXT 또는 DataExchange.UN으로 조회하면, 모든 값은 0 입니다.
+            - 휴장일에는 빈 DataFrame을 반환합니다.
+
         Args:
             asset_code(str): 종목코드
+            data_exchange(DataExchange): 데이터 거래소
 
         Returns:
-            pd.DataFrame: 분봉 데이터
+            pd.DataFrame: 분봉 데이터 (시간의 역순)
         """
 
-        request_time = dtm.datetime.now().replace(second=0, microsecond=0)
+        def _create_minute_dataframe(data: list = None) -> pd.DataFrame:
+            """
+            분봉 데이터를 위한 DataFrame을 생성합니다.
+
+            Args:
+                data (list, optional): 분봉 데이터 리스트. None이면 빈 DataFrame을 반환합니다.
+
+            Returns:
+                pd.DataFrame: 분봉 데이터 DataFrame
+            """
+            if data is None:
+                data = []
+
+            df = pd.DataFrame(data, columns=["time", "open", "high", "low", "close", "volume", "value", "cum_volume", "cum_value"])
+            df["time"] = pd.to_datetime(df["time"], format="%H:%M:%S")
+            df.set_index("time", inplace=True)
+            return df
+
+        request_datetime = dtm.datetime.now()
+        request_time = request_datetime.replace(second=0, microsecond=0)
         result = []
-        schedule = get_market_schedule(dtm.date.today())
+        schedule = get_market_schedule(dtm.date.today(), exchange="KRX" if data_exchange == DataExchange.KRX else "NXT")
+
+        if schedule.full_day_closed:
+            return _create_minute_dataframe()
+
+        # 만약 현재 시간이 거래소 마감 시간을 지났다면, 마감 시간으로 설정
+        if request_time.time() > schedule.close_time:
+            request_time = request_datetime.replace(hour=schedule.close_time.hour, minute=schedule.close_time.minute)
 
         while True:
-            r = self.stock_api.inquire_time_itemchartprice(asset_code, request_time.time(), fid_pw_data_incu_yn='N')
-            output = r['output2']
+            CallLimiter().wait_limit_rate(20, scope="kis/inquire_time_itemchartprice")
+
+            r = self.stock_api.inquire_time_itemchartprice(
+                asset_code,
+                request_time.time(),
+                fid_cond_mrkt_div_code=self._get_data_exchange(data_exchange),
+                fid_pw_data_incu_yn="N",
+            )
+            output = r["output2"]
             if len(output) == 0:
                 break
 
@@ -279,11 +368,12 @@ class KISSimpleDomesticStock:
                     }
                 )
 
-            last_item_time = result[-1]['time']
+            last_item_time = result[-1]["time"]
             request_time = last_item_time - dtm.timedelta(minutes=1)
             if request_time.time() < schedule.open_time:
                 break
 
+        # 거래대금, 누적거래량 계산
         prev_cum_value = None
         prev_cum_volume = None
 
@@ -304,18 +394,19 @@ class KISSimpleDomesticStock:
             prev_cum_value = curr["cum_value"]
             prev_cum_volume = curr["cum_volume"]
 
-        df = pd.DataFrame(result, columns=["time", "open", "high", "low", "close", "volume", "value", "cum_volume", "cum_value"])
-        df['time'] = pd.to_datetime(df['time'], format='%H:%M:%S')
-        df.set_index('time', inplace=True)
+        return _create_minute_dataframe(result)
 
-        return df
-
-    def get_price(self, asset_code: str) -> dict:
+    def get_price(
+        self,
+        asset_code: str,
+        data_exchange: DataExchange = DataExchange.KRX,
+    ) -> dict:
         """
         주식 현재 가격 조회
 
         Args:
             asset_code(str): 종목코드
+            data_exchange(DataExchange): 데이터 거래소
 
         Returns:
             dict: 현재 가격 정보
@@ -331,9 +422,9 @@ class KISSimpleDomesticStock:
             - diff (int): 전일대비
             - diff_rate (float): 전일대비율
         """
-        r = self.stock_api.get_price(asset_code)
+        r = self.stock_api.get_price(asset_code, self._get_data_exchange(data_exchange))
 
-        data = r['output']
+        data = r["output"]
         result = {
             "code": asset_code,
             "current_price": data["stck_prpr"],
@@ -372,7 +463,7 @@ class KISSimpleDomesticStock:
         result = []
 
         for item in r:
-            if item['shcode'] in asset_codes:
+            if item["shcode"] in asset_codes:
                 result.append(
                     {
                         "code": item["shcode"],
@@ -387,17 +478,20 @@ class KISSimpleDomesticStock:
                 )
 
         df = pd.DataFrame(result)
-        df.set_index('code', inplace=True)
+        df.set_index("code", inplace=True)
         return df
 
     @with_mock()
-    def create_order(self,
-                     asset_code: str,
-                     side: OrderSide,
-                     quantity: int,
-                     order_type: OrderType = OrderType.MARKET,
-                     price: int = 0) -> str:
-
+    def create_order(
+        self,
+        asset_code: str,
+        side: OrderSide,
+        quantity: int,
+        order_type: OrderType = OrderType.MARKET,
+        price: int = 0,
+        stop_price: int = 0,
+        exchange: OrderExchange = OrderExchange.KRX,
+    ) -> str:
         """
         주문을 생성합니다.
 
@@ -407,6 +501,8 @@ class KISSimpleDomesticStock:
             quantity (int): 주문 수량
             order_type (OrderType): 주문 유형
             price (int): 주문 가격 (지정가 주문일 경우에만 필요)
+            stop_price (int): 스톱 트리거 가격 (스톱 주문일 경우에만 필요)
+            exchange (OrderExchange): 거래소
 
         Returns:
             str: 주문 번호
@@ -441,6 +537,14 @@ class KISSimpleDomesticStock:
                 return "15"
             elif order_type == OrderType.BEST_PRICE_FOK:
                 return "16"
+            elif order_type == OrderType.MID_POINT:
+                return "21"
+            elif order_type == OrderType.MID_POINT_IOC:
+                return "23"
+            elif order_type == OrderType.MID_POINT_FOK:
+                return "24"
+            elif order_type == OrderType.STOP_LIMIT:
+                return "22"
             else:
                 raise ValueError("지원하지 않는 주문 유형입니다.")
 
@@ -460,17 +564,22 @@ class KISSimpleDomesticStock:
             __get_order_type_code(),
             quantity,
             price,
+            stop_price,
+            self._get_exchange_code(exchange),
         )
 
-        return r['output']['ODNO']
+        return r["output"]["ODNO"]
 
     @with_mock()
-    def update_order(self,
-                     org_order_no: str,
-                     order_type: OrderType,
-                     price: int,
-                     quantity: int = 0) -> str:
-
+    def update_order(
+        self,
+        org_order_no: str,
+        order_type: OrderType,
+        price: int,
+        quantity: int = 0,
+        stop_price: int = 0,
+        exchange: OrderExchange = OrderExchange.KRX,
+    ) -> str:
         """
         주문을 수정합니다.
 
@@ -479,6 +588,8 @@ class KISSimpleDomesticStock:
             order_type (OrderType): 주문 유형
             price (int): 정정 가격
             quantity (int): 주문 수량
+            stop_price (int): 스톱 트리거 가격 (스톱 주문일 경우에만 필요)
+            exchange (OrderExchange): 거래소
 
         Returns:
             str: 주문 번호
@@ -513,53 +624,100 @@ class KISSimpleDomesticStock:
                 return "15"
             elif order_type == OrderType.BEST_PRICE_FOK:
                 return "16"
+            elif order_type == OrderType.MID_POINT:
+                return "21"
+            elif order_type == OrderType.MID_POINT_IOC:
+                return "23"
+            elif order_type == OrderType.MID_POINT_FOK:
+                return "24"
+            elif order_type == OrderType.STOP_LIMIT:
+                return "22"
             else:
                 raise ValueError("지원하지 않는 주문 유형입니다.")
 
-        qty_all_ord_yn = 'Y' if quantity == 0 else 'N'
+        qty_all_ord_yn = "Y" if quantity == 0 else "N"
 
         r = self.stock_api.order_rvsecncl(
             self.account_no,
             self.account_product_code,
             org_order_no,
             __get_order_type_code(),
-            '01', # 정정
+            "01",
             quantity,
             price,
-            qty_all_ord_yn
-        )
+            qty_all_ord_yn,
+            stop_price,
+            self._get_exchange_code(exchange),
+        )  # 정정
 
-        return r['output']['ODNO']
+        return r["output"]["ODNO"]
 
     @with_mock()
-    def cancel_order(self, org_order_no: str, quantity: int = 0) -> str:
+    def cancel_order(
+        self,
+        org_order_no: str,
+        quantity: int = 0,
+        stop_price: int = 0,
+        exchange: OrderExchange = OrderExchange.KRX,
+    ) -> str:
         """
         주문을 취소합니다.
 
         Args:
             org_order_no (str): 원주문번호
             quantity (int): 취소 수량 (일부 취소의 경우 지정)
+            stop_price (int): 스톱 트리거 가격 (스톱 주문일 경우에만 필요)
+            exchange (OrderExchange): 거래소
         """
         r = self.stock_api.order_rvsecncl(
             self.account_no,
             self.account_product_code,
             org_order_no,
-            '00',
-            '02', # 취소
+            "00",
+            "02",
             quantity,
             0,
-            'Y' if quantity == 0 else 'N'
-        )
+            "Y" if quantity == 0 else "N",
+            stop_price,
+            excg_id_dvsn_cd=self._get_exchange_code(exchange),
+        )  # 취소
 
-        return r['output']['ODNO']
+        return r["output"]["ODNO"]
 
-    def get_pending_orders(self) -> List[StockOrder]:
+    def _get_exchange(self, excg_id_dvsn_cd: str) -> OrderExchange:
+        if excg_id_dvsn_cd == "KRX":
+            return OrderExchange.KRX
+        elif excg_id_dvsn_cd == "NXT":
+            return OrderExchange.NXT
+        elif excg_id_dvsn_cd == "SOR":
+            return OrderExchange.SOR
+        else:
+            raise ValueError("지원하지 않는 거래소입니다.")
+
+    def _get_exchange_code(self, exchange: OrderExchange) -> str:
+        if exchange == OrderExchange.KRX:
+            return "KRX"
+        elif exchange == OrderExchange.NXT:
+            return "NXT"
+        elif exchange == OrderExchange.SOR:
+            return "SOR"
+        else:
+            raise ValueError("지원하지 않는 거래소입니다.")
+
+    def get_pending_orders(
+        self,
+        exchanges: List[OrderExchange] = [OrderExchange.KRX],
+    ) -> List[StockOrder]:
         """
         미체결 주문을 조회합니다.
 
+        Args:
+            exchanges (List[OrderExchange]): 거래소 리스트
+
         Returns:
             List[StockOrder]: 미체결 주문 리스트
         """
+
         fetching = True
         ctx_area_fk100 = ""
         ctx_area_nk100 = ""
@@ -578,6 +736,7 @@ class KISSimpleDomesticStock:
                 ctx_area_fk100=ctx_area_fk100,
                 ctx_area_nk100=ctx_area_nk100,
                 tr_cont=tr_cont,
+                excg_id_dvsn_cd="ALL",
             )
 
             output1 = r["output1"]
@@ -586,9 +745,7 @@ class KISSimpleDomesticStock:
                 filled_quantity = int(item["tot_ccld_qty"])
                 filled_amount = int(item["tot_ccld_amt"])
                 pending_quantity = int(item["rmn_qty"])
-                filled_price = (
-                    filled_amount // filled_quantity if filled_quantity > 0 else 0
-                )
+                filled_price = filled_amount // filled_quantity if filled_quantity > 0 else 0
 
                 order = StockOrder(
                     order_no=item["odno"],
@@ -605,6 +762,7 @@ class KISSimpleDomesticStock:
                     org_order_no=item["orgn_odno"],
                     order_type=self._get_order_type(item["ord_dvsn_cd"]),
                     req_type=self._get_req_type(item["sll_buy_dvsn_cd_name"]),
+                    exchange=self._get_exchange(item["excg_id_dvsn_cd"]),
                 )
 
                 if order.org_order_no == "0000000000":
@@ -626,6 +784,9 @@ class KISSimpleDomesticStock:
         for order in result:
             order.current_price = int(price_df.loc[order.asset_code, "current_price"])
 
+        if not self._is_all_exchanges(exchanges):
+            result = [order for order in result if order.exchange in exchanges]
+
         return result
 
     def get_pending_orders_old(self) -> List[StockOrder]:
@@ -653,9 +814,7 @@ class KISSimpleDomesticStock:
             for item in r["output"]:
                 filled_amount = int(item.get("tot_ccld_amt", 0))
                 filled_quantity = int(item.get("tot_ccld_qty", 0))
-                filled_price = (
-                    filled_amount // filled_quantity if filled_quantity > 0 else 0
-                )
+                filled_price = filled_amount // filled_quantity if filled_quantity > 0 else 0
                 asset_code = item["pdno"]
 
                 order = StockOrder(
@@ -671,6 +830,7 @@ class KISSimpleDomesticStock:
                     order_time=item["ord_tmd"],
                     order_type=self._get_order_type(item["ord_dvsn_cd"]),
                     req_type=self._get_req_type(item["rvse_cncl_dvsn_name"]),
+                    exchange=self._get_exchange(item["excg_id_dvsn_cd"]),
                 )
 
                 asset_codes.append(asset_code)
@@ -694,10 +854,16 @@ class KISSimpleDomesticStock:
         self,
         target_date: dtm.date = None,
         order_no: str = "",
+        exchanges: List[OrderExchange] = [OrderExchange.KRX],
     ) -> List[StockOrder]:
         """
         오늘 주문 내역을 조회합니다.
 
+        Args:
+            target_date (dtm.date): 조회할 날짜
+            order_no (str): 조회할 주문 번호
+            exchange (OrderExchange): 거래소
+
         Returns:
             List[StockOrder]: 오늘 주문 내역 리스트
         """
@@ -722,6 +888,7 @@ class KISSimpleDomesticStock:
                 ctx_area_fk100=ctx_area_fk100,
                 ctx_area_nk100=ctx_area_nk100,
                 tr_cont=tr_cont,
+                excg_id_dvsn_cd="ALL",
             )
 
             for item in r["output1"]:
@@ -747,6 +914,7 @@ class KISSimpleDomesticStock:
                     org_order_no=item["orgn_odno"],
                     order_type=self._get_order_type(item["ord_dvsn_cd"]),
                     req_type=self._get_req_type(item["sll_buy_dvsn_cd_name"]),
+                    exchange=self._get_exchange(item["excg_id_dvsn_cd"]),
                 )
                 result.append(order)
 
@@ -757,8 +925,14 @@ class KISSimpleDomesticStock:
             else:
                 fetching = False
 
+        if not self._is_all_exchanges(exchanges):
+            result = [order for order in result if order.exchange in exchanges]
+
         return result
 
+    def _is_all_exchanges(self, exchanges: List[OrderExchange]) -> bool:
+        return set(exchanges) == set(OrderExchange)
+
     def get_order(self, odno: str) -> StockOrder:
         """
         주문 번호로 주문 정보를 조회합니다.
@@ -828,12 +1002,18 @@ class KISSimpleDomesticStock:
             return OrderType.BEST_PRICE_IOC
         elif ord_dvsn_cd == "16":
             return OrderType.BEST_PRICE_FOK
+        elif ord_dvsn_cd == "21":
+            return OrderType.MID_POINT
+        elif ord_dvsn_cd == "22":
+            return OrderType.STOP_LIMIT
+        elif ord_dvsn_cd == "23":
+            return OrderType.MID_POINT_IOC
+        elif ord_dvsn_cd == "24":
+            return OrderType.MID_POINT_FOK
         else:
             raise ValueError(f"지원하지 않는 주문 유형입니다. {ord_dvsn_cd}")
 
-    async def listen_order_event(
-        self, stop_event: Optional[asyncio.Event] = None
-    ) -> AsyncGenerator:
+    async def listen_order_event(self, stop_event: Optional[asyncio.Event] = None) -> AsyncGenerator:
         """
         계좌 주문 이벤트를 수신하는 메서드
 
@@ -916,9 +1096,18 @@ class KISSimpleDomesticStock:
             t = dtm.datetime.strptime(filled_time, "%H%M%S").time()
             filled_time = dtm.datetime.combine(dtm.date.today(), t)
 
-        print(
-            f"- rctf_cls={rctf_cls} acpt_yn={acpt_yn} cntg_yn={cntg_yn} accepted={accepted} executed={executed} rejected={rejected} status={event_type}"
-        )
+        ord_exg_gb = data.get("ord_exg_gb")
+        exchange = None
+        if ord_exg_gb == "1":
+            exchange = OrderExchange.KRX
+        elif ord_exg_gb == "2":
+            exchange = OrderExchange.NXT
+        elif ord_exg_gb == "3":
+            exchange = OrderExchange.SOR
+        elif ord_exg_gb == "4":
+            exchange = OrderExchange.SOR
+
+        print(f"- rctf_cls={rctf_cls} acpt_yn={acpt_yn} cntg_yn={cntg_yn} accepted={accepted} executed={executed} rejected={rejected} status={event_type}")
 
         return OrderEvent(
             asset_code,
@@ -933,14 +1122,16 @@ class KISSimpleDomesticStock:
             filled_price,
             filled_time,
             org_order_no,
+            exchange=exchange,
         )
 
-    def get_orderbook(self, asset_code: str) -> Dict:
+    def get_orderbook(self, asset_code: str, data_exchange: DataExchange = DataExchange.KRX) -> Dict:
         """
         특정 종목의 호가/잔량 정보를 조회하여 반환합니다.
 
         Args:
             asset_code (str): 조회할 자산의 종목번호.
+            data_exchange(DataExchange): 데이터 거래소
 
         Returns:
             dict: 호가 정보가 포함된 사전.
@@ -958,9 +1149,11 @@ class KISSimpleDomesticStock:
             ValueError: 주어진 티커가 유효하지 않거나 찾을 수 없는 경우 발생.
         """
 
-        r = self.stock_api.inquire_asking_price_exp_ccn(asset_code)
+        r = self.stock_api.inquire_asking_price_exp_ccn(asset_code, self._get_data_exchange(data_exchange))
 
         o1 = r["output1"]
+        if not o1:
+            return {}
 
         result = {
             "total_bid_volume": o1["total_bidp_rsqn"],
