@@ -350,7 +350,7 @@ class KISSimpleOverseasStock:
         first_date: dtm.date,
         last_date: dtm.date,
         period: str = "D",
-        adjusted: bool = True,
+        adjusted_price: bool = True,
     ) -> pd.DataFrame:
         """
         특정 티커의 일간 데이터를 조회하여 데이터프레임으로 반환합니다.
@@ -360,7 +360,7 @@ class KISSimpleOverseasStock:
             first_date (datetime.date): 조회를 시작할 첫 번째 날짜 (YYYY-MM-DD). 현지기준 일자.
             last_date (datetime.date): 조회를 마칠 마지막 날짜 (YYYY-MM-DD). 현지 기준 일자.
             period (str, optional): 조회할 데이터의 주기, 기본값은 "D" (일간).
-            adjusted (bool, optional): 수정 종가를 사용할지 여부, 기본값은 True.
+            adjusted_price (bool, optional): 수정 종가를 사용할지 여부, 기본값은 True.
 
         Returns:
             pd.DataFrame: 조회된 일간 데이터가 포함된 Pandas DataFrame.
@@ -397,7 +397,7 @@ class KISSimpleOverseasStock:
         search_date = last_date
 
         gubn = __period_to_gubn(period)
-        modp = "1" if adjusted else "0"
+        modp = "1" if adjusted_price else "0"
 
         while fetching:
             CallLimiter().wait_limit_rate(90, scope="overseas_daily")
@@ -975,44 +975,59 @@ class KISSimpleOverseasStock:
             ValueError: API 호출 중 문제가 발생할 경우 발생.
         """
 
-        r = self.stock_api.inquire_ccnl(
-            self.account_no,
-            self.account_product_code,
-            from_date,
-            to_date,
-            "NASD",
-            "%",
-            "00",
-            "00",
-        )
-        output = r["output"]
-
         result = []
-        for d in output:
-            order_kr_time = dtm.datetime.combine(d["dmst_ord_dt"], d["ord_tmd"], tzinfo=self.kst)
-            order_time = order_kr_time.astimezone(self.nyt)
+        fetching = True
+        ctx_area_fk200 = ""
+        ctx_area_nk200 = ""
+        tr_cont = ""
 
-            order = OverseasStockOrder(
-                order_no=d["odno"],
-                asset_code=d["pdno"],
-                side=OrderSide.BUY if d["sll_buy_dvsn_cd"] == "02" else OrderSide.SELL,
-                price=Decimal(d["ft_ord_unpr3"]),
-                quantity=d["ft_ord_qty"],
-                filled_quantity=d["ft_ccld_qty"],
-                pending_quantity=d["nccs_qty"],
-                order_time=order_time,
-                order_kr_time=order_kr_time,
-                filled_price=d["ft_ccld_unpr3"],
-                current_price=self._get_cached_price(d["pdno"]),
-                is_pending=d["nccs_qty"] != 0,
-                org_order_no=d["orgn_odno"],
-                order_type=None,
-                req_type=self._code_to_order_request_type(d["rvse_cncl_dvsn"]),
-                exchange=d["ovrs_excg_cd"],
-                currency=d["tr_crcy_cd"],
+        while fetching:
+            r = self.stock_api.inquire_ccnl(
+                self.account_no,
+                self.account_product_code,
+                from_date,
+                to_date,
+                "NASD",
+                "%",
+                "00",
+                "00",
+                ctx_area_fk200=ctx_area_fk200,
+                ctx_area_nk200=ctx_area_nk200,
+                tr_cont=tr_cont,
             )
+            output = r["output"]
 
-            result.append(order)
+            for d in output:
+                order_kr_time = dtm.datetime.combine(d["dmst_ord_dt"], d["ord_tmd"], tzinfo=self.kst)
+                order_time = order_kr_time.astimezone(self.nyt)
+
+                order = OverseasStockOrder(
+                    order_no=d["odno"],
+                    asset_code=d["pdno"],
+                    side=OrderSide.BUY if d["sll_buy_dvsn_cd"] == "02" else OrderSide.SELL,
+                    price=Decimal(d["ft_ord_unpr3"]),
+                    quantity=d["ft_ord_qty"],
+                    filled_quantity=d["ft_ccld_qty"],
+                    pending_quantity=d["nccs_qty"],
+                    order_time=order_time,
+                    order_kr_time=order_kr_time,
+                    filled_price=d["ft_ccld_unpr3"],
+                    current_price=self._get_cached_price(d["pdno"]),
+                    is_pending=d["nccs_qty"] != 0,
+                    org_order_no=d["orgn_odno"],
+                    order_type=None,
+                    req_type=self._code_to_order_request_type(d["rvse_cncl_dvsn"]),
+                    exchange=d["ovrs_excg_cd"],
+                    currency=d["tr_crcy_cd"],
+                )
+                result.append(order)
+
+            if r["tr_cont"] in ["F", "M"]:
+                ctx_area_fk200 = r["ctx_area_fk200"]
+                ctx_area_nk200 = r["ctx_area_nk200"]
+                tr_cont = "N"
+            else:
+                fetching = False
 
         if to_frame:
             rows = []
