@@ -1,29 +1,42 @@
-from typing import Dict
+import datetime
+from typing import Dict, Union
+
+import numpy as np
+import pandas as pd
+import pytz
+
+import pyqqq.config as c
+from pyqqq.datatypes import DataExchange
 from pyqqq.utils.api_client import raise_for_status, send_request
 from pyqqq.utils.local_cache import DiskCacheManager
 from pyqqq.utils.logger import get_logger
-import datetime as dtm
-import numpy as np
-import pandas as pd
-import pyqqq.config as c
-import pytz
-
 
 logger = get_logger(__name__)
 minuteCache = DiskCacheManager("minute_cache")
 
 
 @minuteCache.memoize()
-def get_all_minute_data(time: dtm.datetime, source: str = "ebest", adjusted: bool = True) -> pd.DataFrame:
+def get_all_minute_data(
+    time: datetime.datetime,
+    source: str = "kis",
+    adjusted: bool = True,
+    exchange: Union[str, DataExchange] = "KRX",
+) -> pd.DataFrame:
     """
     모든 종목의 분봉 데이터를 반환합니다.
 
     2024년 4월 9일 데이터 부터 조회 가능합니다.
 
+    NXT 거래소 데이터의 조회 가능 시작일은 데이터 소스에 따라 다릅니다. kis는 2025년 3월 4일부터, ebest는 2025년 5월 12일부터 데이터를 조회할 수 있습니다.
+
+    Note:
+        - KRX, NXT 거래소의 분봉 데이터를 조회할 수 있습니다. UN 거래소는 지원되지 않습니다.
+
     Args:
-        time (dtm.datetime): 조회할 시간
-        source (str): 데이터를 검색할 API. 'ebest' 또는 'kis'를 지정할 수 있습니다. 기본값은 'ebest'입니다.
+        time (datetime.datetime): 조회할 시간
+        source (str): 데이터를 검색할 API. 'ebest' 또는 'kis'를 지정할 수 있습니다. 기본값은 'kis'입니다.
         adjusted (bool): 수정주가 여부. 기본값은 True.
+        exchange (Union[str, DataExchange]): 거래소. 기본값은 KRX.
 
     Returns:
         pd.DataFrame: 모든 종목의 분봉 데이터가 포함된 pandas DataFrame.
@@ -53,8 +66,11 @@ def get_all_minute_data(time: dtm.datetime, source: str = "ebest", adjusted: boo
         - mdvolumetm (int): 시간별매도체결량
         - msvolumetm (int): 시간별매수체결량
 
+    Raises:
+        ValueError: 지원하지 않는 거래소 코드가 전달된 경우.
+
     Examples:
-        >>> df = get_all_minute_data(dtm.datetime(2024, 5, 2, 15, 30))
+        >>> df = get_all_minute_data(datetime.datetime(2024, 5, 2, 15, 30))
         >>> print(df)
                                     time   open   high    low  ...  totofferrem  totbidrem mdvolumetm  msvolumetm
         code                                             ...
@@ -68,11 +84,16 @@ def get_all_minute_data(time: dtm.datetime, source: str = "ebest", adjusted: boo
     """
     tz = pytz.timezone("Asia/Seoul")
 
+    exchange = DataExchange.validate(exchange)
+    if exchange == DataExchange.UN:
+        raise ValueError("UN 거래소는 지원되지 않습니다.")
+
     url = f"{c.PYQQQ_API_URL}/domestic-stock/ohlcv/minutes/all/{time.date()}/{time.strftime('%H%M')}"
     params = {
         "brokerage": source,
         "adjusted": "true" if adjusted else "false",
-        "current_date": dtm.date.today(),
+        "current_date": datetime.date.today(),
+        "exchange": exchange.value,
     }
 
     r = send_request("GET", url, params=params)
@@ -83,7 +104,7 @@ def get_all_minute_data(time: dtm.datetime, source: str = "ebest", adjusted: boo
     rows = r.json()
     for data in rows:
         time = data["time"].replace("Z", "+00:00")
-        time = dtm.datetime.fromisoformat(time).astimezone(tz).replace(tzinfo=None)
+        time = datetime.datetime.fromisoformat(time).astimezone(tz).replace(tzinfo=None)
         data["time"] = time
 
     df = pd.DataFrame(rows)
@@ -115,29 +136,36 @@ def get_all_minute_data(time: dtm.datetime, source: str = "ebest", adjusted: boo
 
 @minuteCache.memoize()
 def get_all_day_data(
-    date: dtm.date,
+    date: datetime.date,
     codes: list[str] | str,
-    period: dtm.timedelta = dtm.timedelta(minutes=1),
-    source: str = "ebest",
+    period: datetime.timedelta = datetime.timedelta(minutes=1),
+    source: str = "kis",
     adjusted: bool = True,
     ascending: bool = True,
+    exchange: Union[str, DataExchange] = "KRX",
 ) -> dict[str, pd.DataFrame] | pd.DataFrame:
     """
     지정된 날짜에 대해 하나 이상의 주식 코드에 대한 전체 분별 OHLCV(시가, 고가, 저가, 종가, 거래량) 데이터를 검색하여 반환합니다.
 
     2024년 4월 26일 데이터 부터 조회 가능합니다.
 
+    NXT 거래소 데이터의 조회 가능 시작일은 데이터 소스에 따라 다릅니다. kis는 2025년 3월 4일부터, ebest는 2025년 5월 12일부터 데이터를 조회할 수 있습니다.
+
+    Note:
+        - KRX, NXT 거래소의 분봉 데이터를 조회할 수 있습니다. UN 거래소는 지원되지 않습니다.
+
     Args:
-        date (dtm.date): 데이터를 검색할 날짜.
+        date (datetime.date): 데이터를 검색할 날짜.
         codes (list[str]): 조회할 주식 코드들의 리스트. 최대 20개까지 지정할 수 있습니다.
-        period (dtm.timedelta, optional): 반환된 데이터의 시간 간격. 기본값은 1분입니다. 30초 이상의 값을 30초간격으로 지정할 수 있습니다.
-        source (str, optional): 데이터를 검색할 API. 'ebest' 또는 'kis'를 지정할 수 있습니다. 기본값은 'ebest'입니다.
+        period (datetime.timedelta, optional): 반환된 데이터의 시간 간격. 기본값은 1분입니다. 30초 이상의 값을 30초간격으로 지정할 수 있습니다.
+        source (str, optional): 데이터를 검색할 API. 'ebest' 또는 'kis'를 지정할 수 있습니다. 기본값은 'kis'입니다.
         adjusted (bool): 수정주가 여부. 기본값은 True.
         ascending (bool): 오름차순 여부. 기본값은 True.
+        exchange (Union[str, DataExchange]): 거래소. 기본값은 KRX. (cf. NXT의 경우 해당되지 않는 종목은 Empty DataFrame이 반환됩니다.)
 
     Returns:
         dict[str, pd.DataFrame]: 주식 코드를 키로 하고, 해당 주식의 일일 OHLCV 데이터가 포함된 pandas DataFrame을 값으로 하는 딕셔너리.
-        각 DataFrame에는 변환된 'time' 열이 포함되어 있으며, 이는 조회된 데이터의 시간을 나타냅니다. 'time' 열은 DataFrame의 인덱스로 설정됩니다.
+        각 DataFrame에는 변환된 'time' 열이 포함되어 있으며, 이는 조회된 데이터의 시간을 나타냅니다. 'time' 열은 DataFrame의 인덱스로 설정되고 오름차순으로 정렬됩니다.
 
         DataFrame의 열은 다음과 같습니다:
 
@@ -168,7 +196,7 @@ def get_all_day_data(
         requests.exceptions.RequestException: PYQQQ API로부터 데이터를 검색하는 과정에서 오류가 발생한 경우.
 
     Examples:
-        >>> result = get_all_day_data(dtm.date(2024, 4, 26), ["005930", "319640"], dtm.timedelta(minutes=1))
+        >>> result = get_all_day_data(datetime.date(2024, 4, 26), ["005930", "319640"], datetime.timedelta(minutes=1))
         >>> print(result["069500"])
                             open   high    low  close   volume sign  change  diff  \
         time
@@ -178,7 +206,8 @@ def get_all_day_data(
         2024-04-26 09:03:00  77500  77500  77200  77500  3033307    2    1200  1.57
         2024-04-26 09:04:00  77400  77600  77400  77500  3268502    2    1200  1.57
     """
-    assert isinstance(date, dtm.date), "date must be a datetime.date object"
+    assert isinstance(date, datetime.date), "date must be a datetime.date object"
+    assert type(date) is datetime.date, "date must be a datetime.date object"
     assert isinstance(codes, list) or isinstance(codes, str), "codes must be a list of strings or single code"
 
     if isinstance(codes, list):
@@ -187,13 +216,16 @@ def get_all_day_data(
     assert len(codes) <= 20, "codes must not exceed 20"
 
     if period is not None:
-        assert period >= dtm.timedelta(seconds=30), "period must be at least 30 seconds"
+        assert period >= datetime.timedelta(seconds=30), "period must be at least 30 seconds"
         assert period.total_seconds() % 30 == 0, "period must be a multiple of 30 seconds"
 
     tz = pytz.timezone("Asia/Seoul")
     target_codes = codes if isinstance(codes, list) else [codes]
 
-    if source == "kis":
+    exchange = DataExchange.validate(exchange)
+    if exchange == DataExchange.UN:
+        raise ValueError("UN 거래소는 지원되지 않습니다.")
+    elif exchange == DataExchange.NXT or source == "kis":
         url = f"{c.PYQQQ_API_URL}/domestic-stock/ohlcv/minutes/{date}"
     else:
         url = f"{c.PYQQQ_API_URL}/domestic-stock/ohlcv/half-minutes/{date}"
@@ -205,7 +237,8 @@ def get_all_day_data(
             "codes": ",".join(target_codes) if target_codes else None,
             "brokerage": source,
             "adjusted": "true" if adjusted else "false",
-            "current_date": dtm.date.today(),
+            "current_date": datetime.date.today(),
+            "exchange": exchange.value,
         },
     )
 
@@ -229,7 +262,7 @@ def get_all_day_data(
         rows = multirows[code]
         for row in rows:
             time = row[time_index].replace("Z", "+00:00")
-            time = dtm.datetime.fromisoformat(time).astimezone(tz).replace(tzinfo=None)
+            time = datetime.datetime.fromisoformat(time).astimezone(tz).replace(tzinfo=None)
             row[time_index] = time
 
         rows.reverse()
@@ -253,7 +286,7 @@ def get_all_day_data(
 
 def resample_ebest_data(df, period):
     if period is not None and period.total_seconds() != 30:
-        df["time"] = df["time"] - dtm.timedelta(seconds=30)
+        df["time"] = df["time"] - datetime.timedelta(seconds=30)
         df.set_index("time", inplace=True)
 
         minutes = period.total_seconds() / 60
@@ -312,6 +345,7 @@ def resample_ebest_data(df, period):
 
 def resample_kis_data(df, period):
     if period is not None and period.total_seconds() != 60:
+        df.set_index("time", inplace=True)
         minutes = period.total_seconds() // 60
 
         op_dict = {
@@ -350,13 +384,13 @@ def resample_kis_data(df, period):
     return df
 
 
-def get_orderbook(code: str, time: dtm.datetime) -> Dict:
+def get_orderbook(code: str, time: datetime.datetime) -> Dict:
     """
     주식 종목의 주문 호가 정보를 반환합니다.
 
     Args:
         code (str): 종목 코드
-        time (dtm.datetime): 조회할 시간
+        time (datetime.datetime): 조회할 시간
 
     Returns:
         dict: 호가 정보가 포함된 사전.
@@ -366,7 +400,7 @@ def get_orderbook(code: str, time: dtm.datetime) -> Dict:
             - ask_volume (int): 1차 매도 호가 잔량.
             - bid_price (int): 1차 매수 호가 가격.
             - bid_volume (int): 1차 매수 호가 잔량.
-            - time (dtm.datetime): 현지 기준 호가 정보 조회 시간.
+            - time (datetime.datetime): 현지 기준 호가 정보 조회 시간.
             - bids (list): 매수 호가 목록 (각 항목은 price와 volume을 포함하는 dict).
             - asks (list): 매도 호가 목록 (각 항목은 price과 volume을 포함하는 dict).
     """
@@ -380,6 +414,6 @@ def get_orderbook(code: str, time: dtm.datetime) -> Dict:
 
     data = r.json()
     data.pop("code")
-    data["time"] = dtm.datetime.fromisoformat(data["time"]).astimezone(pytz.timezone("Asia/Seoul")).replace(tzinfo=None)
+    data["time"] = datetime.datetime.fromisoformat(data["time"]).astimezone(pytz.timezone("Asia/Seoul")).replace(tzinfo=None)
 
     return data
