@@ -1,39 +1,44 @@
-from typing import List
+import asyncio
+import datetime as dtm
+import inspect
+import json
+import os
+from typing import List, Union
+
+import websockets
+
+import pyqqq.config as c
+from pyqqq.datatypes import DataExchange
 from pyqqq.utils.api_client import raise_for_status, send_request
 from pyqqq.utils.logger import get_logger
 from pyqqq.utils.singleton import singleton
-import pyqqq.config as c
-import asyncio
-import os
-import websockets
-import json
-import inspect
-import datetime as dtm
-
 
 logger = get_logger(__name__)
 
 
-def get_all_last_trades(codes: List[str] = None):
+def get_all_last_trades(
+    codes: List[str] = None,
+    exchange: Union[str, DataExchange] = "KRX",
+):
     """
     모든 종목의 최근 체결 정보를 반환합니다.
 
+    Note:
+        - KRX, UN 거래소의 최근 체결 정보를 조회할 수 있습니다. NXT 거래소는 지원되지 않습니다.
+
     Args:
         codes (List[str], optional): 종목 코드 리스트. 모든 종목의 체결 정보를 반환하려면 None을 전달합니다.
+        exchange (str): 거래소. 기본값은 "KRX"입니다. 지원하는 거래소는 "KRX", "UN" 입니다.
 
     Returns:
         list:
         - dict:
-            - chetime (str): 체결시간
-            - sign (str): 전일대비구분
+            - sign (int): 전일대비구분
             - change (int): 전일대비가격
             - drate (float): 전일대비등락율
             - price (int): 체결가
-            - opentime (str): 시가시간
             - open (int): 시가
-            - hightime (str): 고가시간
             - high (int): 고가
-            - lowtime (str): 저가시간
             - low (int): 저가
             - cgubun (str): 체결구분
             - cvolume (int): 체결량
@@ -49,12 +54,20 @@ def get_all_last_trades(codes: List[str] = None):
             - status (str): 장정보
             - jnilvolume (int): 전일동시간대거래량
             - shcode (str): 종목코드
+            - exchname (str): 거래소명
+            - date (str): 체결일자
+            - update_time (str): 업데이트 시간
 
+    Raises:
+        ValueError: 지원하지 않는 거래소 코드가 전달된 경우.
     """
+    exchange = DataExchange.validate(exchange)
+    if exchange == DataExchange.NXT:
+        raise ValueError("NXT 거래소는 지원되지 않습니다.")
 
-    params = None
+    params = {"exchange": exchange.value}
     if codes:
-        params = {"codes": ",".join(codes) if isinstance(codes, list) else codes}
+        params["codes"] = ",".join(codes) if isinstance(codes, list) else codes
 
     r = send_request("GET", f"{c.PYQQQ_API_URL}/domestic-stock/trades", params=params)
     raise_for_status(r)
@@ -95,6 +108,7 @@ def get_all_last_orderbooks():
 
     return result
 
+
 @singleton
 class TickEventListener:
     """
@@ -144,10 +158,7 @@ class TickEventListener:
         """
         틱데이터 이벤트 리스너를 시작합니다
         """
-        self.tasks = [
-            asyncio.create_task(self.connect_ws()),
-            asyncio.create_task(self.health_ping())
-        ]
+        self.tasks = [asyncio.create_task(self.connect_ws()), asyncio.create_task(self.health_ping())]
 
     async def stop(self):
         """
@@ -203,24 +214,38 @@ class TickEventListener:
         await asyncio.sleep(self.retry_cnt * 2)
         await self.connect_ws()
 
-    def append_event(self, ticker, event_id, price, once, side, price_comparison, listen_callback):
+    def append_event(self, ticker, event_id, price, once, side, price_comparison, listen_callback, is_unified=False):
         """
         사용자로부터 틱데이터 이벤트 추가 요청을 등록합니다. 즉시 처리되진 않습니다.
+
+        Args:
+            ticker (str): 종목 코드
+            event_id (str): 이벤트 ID. 고유한 값이어야 합니다.
+            price (int): 가격
+            once (bool): 한번만 실행할지 여부
+            side (int | OrderSide): 매도/매수 구분 (0: all, 1: sell[cgubun: "-"], 2: buy[cgubun: "+"]) default: 0
+            price_comparison (str): 가격 비교 방식 ("<", "<=", "==", ">=", ">")
+            listen_callback (callable): 이벤트 발생시 호출할 콜백 함수
+            is_unified (bool): 통합 이벤트인지 여부. 기본값은 False입니다.
         """
-        self.events[event_id] = TickEvent(ticker=ticker, event_listener=self, client_id=self.client_id, event_id=event_id, price=price, once=once, side=side, price_comparison=price_comparison, listen_callback=listen_callback)
+        self.events[event_id] = TickEvent(
+            ticker=ticker, event_listener=self, client_id=self.client_id, event_id=event_id, price=price, once=once, side=side, price_comparison=price_comparison, listen_callback=listen_callback, is_unified=is_unified
+        )
         self.pending_event_ids.append(event_id)
 
-    async def append_event_async(self, ticker, event_id, price, once, side, price_comparison, listen_callback):
+    async def append_event_async(self, ticker, event_id, price, once, side, price_comparison, listen_callback, is_unified=False):
         """
         사용자로부터 틱데이터 이벤트 추가 요청을 등록합니다. 비동기로 가능한 한 즉시 처리됩니다.
         """
         if self.ws_connected:
-            self.events[event_id] = TickEvent(ticker=ticker, event_listener=self, client_id=self.client_id, event_id=event_id, price=price, once=once, side=side, price_comparison=price_comparison, listen_callback=listen_callback)
+            self.events[event_id] = TickEvent(
+                ticker=ticker, event_listener=self, client_id=self.client_id, event_id=event_id, price=price, once=once, side=side, price_comparison=price_comparison, listen_callback=listen_callback, is_unified=is_unified
+            )
             logger.info(f"{self.LOG_TAG}append_event_async {event_id}")
             await self.send_subscribe(event_id)
         else:
             logger.info(f"{self.LOG_TAG}append_event_async {event_id} failed")
-            self.append_event(ticker, event_id, price, once, side, price_comparison, listen_callback)
+            self.append_event(ticker, event_id, price, once, side, price_comparison, listen_callback, is_unified)
 
     async def close_event(self, event_id):
         """
@@ -342,7 +367,7 @@ class TickEvent:
     LOG_TAG = "[TickEvent]"
     CLOSE_TIME = dtm.time(18, 0)
 
-    def __init__(self, event_listener, ticker, client_id, event_id, price, once, side, price_comparison, listen_callback):
+    def __init__(self, event_listener, ticker, client_id, event_id, price, once, side, price_comparison, listen_callback, is_unified=False):
         logger.info(f"{self.LOG_TAG} create {event_id}, {ticker}, {price}, {price_comparison}")
         self.removed = False
         self.event_listener = event_listener
@@ -354,20 +379,21 @@ class TickEvent:
         self.side = side
         self.price_comparison = price_comparison
         self.listen_callback = listen_callback
+        self.is_unified = is_unified  # 통합 이벤트인지 여부
 
         self.date = dtm.datetime.now().strftime("%Y%m%d")
 
-    async def listen_tick_event(self, ws):
-        logger.info(f"{self.LOG_TAG} listen {self.event_id}")
-        self.removed = False
-        await ws.send(json.dumps({"type": "subscribe", "ticker": self.ticker, "client_id": self.client_id, "event_id": self.event_id, "price": self.price, "once": self.once, "side": self.side, "price_comparison": self.price_comparison}))
-
     def get_subscribe_dump_data(self):
         logger.info(f"{self.LOG_TAG} subscribe {self.event_id}, {self.ticker}, {self.price}, {self.price_comparison}, {self.side}")
-        return json.dumps({"type": "subscribe", "ticker": self.ticker, "client_id": self.client_id, "event_id": self.event_id, "price": self.price, "once": self.once, "side": self.side, "price_comparison": self.price_comparison})
+
+        ticker = "U" + self.ticker if self.is_unified else self.ticker
+
+        return json.dumps({"type": "subscribe", "ticker": ticker, "client_id": self.client_id, "event_id": self.event_id, "price": self.price, "once": self.once, "side": self.side, "price_comparison": self.price_comparison})
 
     def get_unsubscribe_dump_data(self):
-        return json.dumps({"type": "unsubscribe", "ticker": self.ticker, "client_id": self.client_id, "event_id": self.event_id})
+        ticker = "U" + self.ticker if self.is_unified else self.ticker
+
+        return json.dumps({"type": "unsubscribe", "ticker": ticker, "client_id": self.client_id, "event_id": self.event_id})
 
     async def handle_tick_data(self, data):
         if self.listen_callback:
