@@ -1,8 +1,47 @@
 from dataclasses import dataclass
 from enum import Enum
 from decimal import Decimal
-from typing import Optional
-import datetime as dtm
+from typing import Optional, Union
+import datetime
+
+
+class Exchange(Enum):
+    KRX = "KRX"
+    """ 한국거래소 """
+    NXT = "NXT"
+    """ 넥스트레이드 """
+    NYSE = "NYSE"
+    """ 뉴욕거래소 """
+
+
+class DataExchange(Enum):
+    KRX = "KRX"
+    """ 한국거래소 """
+    NXT = "NXT"
+    """ 넥스트레이드 """
+    UN = "UN"
+    """ 통합 """
+
+    @classmethod
+    def validate(cls, exchange: Union[str, "DataExchange"]) -> "DataExchange":
+        if isinstance(exchange, cls):
+            return exchange
+        if isinstance(exchange, str):
+            try:
+                return cls(exchange)
+            except ValueError:
+                raise ValueError(f"지원하지 않는 거래소 코드입니다: {exchange}")
+
+        raise TypeError(f"exchange는 str 또는 DataExchange 타입이어야 합니다. 현재 타입: {type(exchange)}")
+
+
+class OrderExchange(Enum):
+    KRX = 1
+    """ 한국거래소 """
+    NXT = 2
+    """ 넥스트레이드 """
+    SOR = 3
+    """ 자동주문전송 """
 
 
 class OrderSide(Enum):
@@ -55,6 +94,14 @@ class OrderType(Enum):
     """ Market On Close, 해외 주식 주문 유형 (매도시에만 가능) """
     LOC = 34
     """ Limit On Close, 해외 주식 주문 유형 """
+    MID_POINT = 21
+    """ 중간가 """
+    MID_POINT_IOC = 23
+    """ 중간가, 즉시 (일부)체결 그리고 잔량취소 """
+    MID_POINT_FOK = 24
+    """ 중간가, 즉시 전량체결 또는 전량취소 """
+    STOP_LIMIT = 22
+    """ 스톱 지정가 """
 
 
 class OrderRequestType(Enum):
@@ -131,7 +178,7 @@ class StockOrder:
     """ 체결 수량 """
     pending_quantity: int
     """ 미체결 수량 """
-    order_time: dtm.datetime
+    order_time: datetime.datetime
     """ 주문 시각 """
     filled_price: Optional[int | Decimal] = None
     """ 체결 가격 """
@@ -145,6 +192,8 @@ class StockOrder:
     """ 주문 유형 """
     req_type: Optional[OrderRequestType] = None
     """ 주문 요청 유형 """
+    exchange: Optional[OrderExchange] = None
+    """ 거래소 """
 
 
 @dataclass(kw_only=True)
@@ -159,7 +208,7 @@ class OverseasStockOrder(StockOrder):
     """ 통화 """
     reject_reason: Optional[str] = None
     """ 거부 사유 """
-    order_kr_time: Optional[dtm.datetime] = None
+    order_kr_time: Optional[datetime.datetime] = None
     """ 한국 기준 주문 시각 """
 
 
@@ -185,12 +234,14 @@ class OrderEvent:
     """ 체결 수량 """
     filled_price: Optional[int] = None
     """ 체결 가격 """
-    filled_time: Optional[dtm.datetime] = None
+    filled_time: Optional[datetime.datetime] = None
     """ 체결 시각 """
     org_order_no: Optional[str] = None
     """ 원주문번호 """
     average_purchase_price: Optional[Decimal] = None
     """ 평균 매입 가격 """
+    exchange: Optional[OrderExchange] = None
+    """ 거래소 """
 
     @staticmethod
     def with_pending_order(account_no: str, order: StockOrder):
@@ -214,6 +265,7 @@ class OrderEvent:
             filled_price=order.filled_price,
             filled_time=None,
             org_order_no=order.org_order_no,
+            exchange=order.exchange,
         )
 
 
